/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cstdio>
#include <cstring>
#include "conffile.h"
#include "i18n.h"

Config::ConfFile::ConfFile() throw()
{
}

Config::ConfFile::~ConfFile()
{
}

void
Config::ConfFile::load(const char *fn) throw(Error::ConfError)
{
	const int BufLen = 80;
	char buf[BufLen], auxBuf[BufLen];

	std::FILE *in = fopen(fn, "r");
	if(!in)
		throw Error::ConfError("ConfFile::load", _("Configuration file not found"));

	while(!std::feof(in)) {
		int i;
		bool last = false;

		if(!std::fgets(buf, BufLen, in))
			break;
		if(char *p = std::strchr(buf, '\n'))
			*p = 0, last = true;

		line.push_back(std::string(buf));

		if(buf[0] != '#') {
			char *p = std::strchr(buf, '=');
			if(p == buf || p == 0)
				continue;

			*p = 0;
			table[std::string(buf)].linNum = line.size() - 1;
			table[std::string(buf)].content = std::string(p + 1);
		}

		while(!last && !std::feof(in)) {
			fgets(auxBuf, BufLen, in);
			if(char *p = std::strchr(auxBuf, '\n'))
				*p = 0, last = true;

			line.back() += auxBuf;
			if(buf[0] != '#')
				table[std::string(buf)].content += auxBuf;
		}
	}

	std::fclose(in);
}

void
Config::ConfFile::save(const char *fn) throw()
{
	std::FILE *out = fopen(fn, "w");
	if(!out)
		throw Error::ConfError("ConfFile::save", _("Configuration file cannot be saved"));

	for(vector<std::string>::iterator i = line.begin(); i != line.end(); ++i)
		std::fprintf(out, "%s\n", (*i).c_str());

	std::fclose(out);
}

bool
Config::ConfFile::getValue(const char *key, std::string &val) throw()
{
	map<std::string, Value>::iterator m = table.find(std::string(key));
	if(m != table.end())
		return val = m->second.content, true;
	else
		return val = std::string(), false;
}

bool
Config::ConfFile::setValue(const char *key, const std::string &val) throw()
{
	map<std::string, Value>::iterator m = table.find(std::string(key));
	if(m != table.end()) {
		m->second.content = val;
		line[m->second.linNum] = std::string(key) + '=' + val;
	} else {
		line.push_back(std::string(key) + '=' + val);
		table[std::string(key)].linNum = line.size() - 1;
		table[std::string(key)].content = val;
	}

	return true;
}

