/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <gnome.h>
#include <math.h>
#include "guituner.h"
#include "gnomegui.h"
#include "prefwindow.h"
#include "support.h"

namespace Interface {
	void onExitActivate(GtkMenuItem *, gpointer);
	void onPreferencesActivate(GtkMenuItem *, gpointer);
	void onAboutActivate(GtkMenuItem *, gpointer);

	gint delete_event(GtkWidget *, GdkEvent *, gpointer);
	gint expose_event(GtkWidget *, GdkEventExpose *, gpointer);

	gint idle_loop(gpointer);

	GnomeUIInfo file_menu_uiinfo[] = {
		GNOMEUIINFO_MENU_EXIT_ITEM(onExitActivate, NULL),
		GNOMEUIINFO_END
	};

	GnomeUIInfo settings_menu_uiinfo[] = {
		GNOMEUIINFO_MENU_PREFERENCES_ITEM(onPreferencesActivate, NULL),
		GNOMEUIINFO_END
	};

	GnomeUIInfo help_menu_uiinfo[] = {
		GNOMEUIINFO_MENU_ABOUT_ITEM(onAboutActivate, NULL),
		GNOMEUIINFO_END
	};

	GnomeUIInfo menuBar_uiinfo[] = {
		GNOMEUIINFO_MENU_FILE_TREE(file_menu_uiinfo),
		GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu_uiinfo),
		GNOMEUIINFO_MENU_HELP_TREE(help_menu_uiinfo),
		GNOMEUIINFO_END
	};
}

void
Interface::GnomeGui::initMenu() throw()
{
	GtkWidget *menuBar = gtk_menu_bar_new();
	gtk_widget_ref(menuBar);
	gtk_object_set_data_full(GTK_OBJECT(mainWindow), "menuBar", menuBar,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(menuBar);
	gtk_box_pack_start(GTK_BOX(vBox), menuBar, FALSE, FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(menuBar), 2);
	gnome_app_fill_menu(GTK_MENU_SHELL(menuBar), menuBar_uiinfo,
		NULL, FALSE, 0);

	gtk_widget_ref(menuBar_uiinfo[0].widget);
	gtk_object_set_data_full(GTK_OBJECT(mainWindow), "file1",
		menuBar_uiinfo[0].widget, (GtkDestroyNotify) gtk_widget_unref);

	gtk_widget_ref(file_menu_uiinfo[0].widget);
	gtk_object_set_data_full(GTK_OBJECT(mainWindow), "exit",
		file_menu_uiinfo[0].widget,
		(GtkDestroyNotify) gtk_widget_unref);

	gtk_widget_ref(menuBar_uiinfo[1].widget);
	gtk_object_set_data_full(GTK_OBJECT (mainWindow), "settings",
		menuBar_uiinfo[1].widget,
		(GtkDestroyNotify) gtk_widget_unref);

	gtk_widget_ref(settings_menu_uiinfo[0].widget);
	gtk_object_set_data_full(GTK_OBJECT (mainWindow), "preferences",
		settings_menu_uiinfo[0].widget,
		(GtkDestroyNotify) gtk_widget_unref);

	gtk_widget_ref(menuBar_uiinfo[2].widget);
	gtk_object_set_data_full(GTK_OBJECT (mainWindow), "help1",
		menuBar_uiinfo[2].widget,
		(GtkDestroyNotify) gtk_widget_unref);

	gtk_widget_ref(help_menu_uiinfo[0].widget);
	gtk_object_set_data_full(GTK_OBJECT(mainWindow), "about1",
		help_menu_uiinfo[0].widget,
		(GtkDestroyNotify) gtk_widget_unref);
}

void
Interface::GnomeGui::initDisplay() throw()
{
	pix = gdk_pixmap_new(mainWindow->window, width, height, -1);
	back = gdk_pixmap_new(mainWindow->window, width, height, -1);
	loadFiles();
	const double K = (M_PI/2500) * 14;
	const char *fontName = "-Adobe-Utopia-Regular-R-Normal--19-180-75-75-P-101-iso8859-1";

	int x = width / 2;
	int y = height / 2 + 75;

	font = gdk_font_load(fontName);

	for(int i = 0; i < 101; ++i) {
		ends[i].x1 = static_cast<int>(x + 149 * sin(K * (i - 50)));
		ends[i].y1 = static_cast<int>(y - 149 * cos(K * (i - 50)));
		ends[i].x = static_cast<int>(x + 65 * sin(K * (i - 50)));
		ends[i].y = static_cast<int>(y - 65 * cos(K * (i - 50)));
	}

	gdk_draw_rectangle(back, mainWindow->style->black_gc, TRUE, 0, 0, width, height);
	x = (width - 385) / 2;
	y = height / 2 - 100;
	gdk_draw_pixmap(back, mainWindow->style->fg_gc[GTK_WIDGET_STATE(mainWindow)], file[0],
		0, 0, x, y, 380, 200);

	x = width / 2;
	y = height - 60;
	gdk_draw_pixmap(back, mainWindow->style->fg_gc[GTK_WIDGET_STATE(mainWindow)],
		file[1], 0, 0, x - 120, y, 50, 50);
	gdk_draw_pixmap(back, mainWindow->style->fg_gc[GTK_WIDGET_STATE(mainWindow)],
		file[2], 0, 0, x + 70, y, 50, 50);
	gdk_draw_pixmap(pix, mainWindow->style->fg_gc[GTK_WIDGET_STATE(mainWindow)],
		back, 0, 0, 0, 0, width, height);
}

void
Interface::GnomeGui::addHandlers() throw()
{
	gtk_signal_connect(GTK_OBJECT(mainWindow), "delete_event",
		GTK_SIGNAL_FUNC(delete_event), NULL);
	gtk_signal_connect(GTK_OBJECT(mainWindow), "destroy_event",
		GTK_SIGNAL_FUNC(delete_event), NULL);
	gtk_signal_connect(GTK_OBJECT(drawingArea), "expose_event",
		GTK_SIGNAL_FUNC(expose_event), this);
}

void
Interface::GnomeGui::loadFiles() throw()
{
	GdkBitmap *mask;
	GdkColormap *colorMap;
	gchar *pathName;

	const char *fName[5] = {
		"GuiTuner/back.xpm",
		"GuiTuner/arr01.xpm",
		"GuiTuner/arr02.xpm",
		"GuiTuner/arr03.xpm",
		"GuiTuner/arr04.xpm"
	};

	colorMap = gtk_widget_get_colormap(mainWindow);
	for(int i = 0; i < 5; ++i) {
		pathName = gnome_pixmap_file(fName[i]);
		file[i] = gdk_pixmap_colormap_create_from_xpm(NULL, colorMap, &mask,
			NULL, pathName);
		g_free(pathName);
		if(file[i]);
			gdk_bitmap_unref(mask);
	}
}

void
Interface::GnomeGui::destroy() throw()
{
	for(int i = 0; i < 5; ++i)
		gdk_pixmap_unref(file[i]);
	gdk_pixmap_unref(back);
	gdk_pixmap_unref(pix);
	gtk_widget_destroy(mainWindow);
}

Interface::GnomeGui::GnomeGui(int *argc, char ***argv) throw(): loop(false),
	isOpen(false), curr(0)
{
	gnome_init("GuiTuner", VERSION, *argc, *argv);
}

Interface::GnomeGui::~GnomeGui()
{
	close();
}

void
Interface::GnomeGui::open() throw(Error::GuiError)
{
	GtkWidget *logo;

	mainWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_usize(mainWindow, 400, 290);
	gtk_container_set_border_width(GTK_CONTAINER(mainWindow), 2);
	gtk_window_set_title(GTK_WINDOW(mainWindow), _("GuiTuner 0.05"));
	gtk_window_set_policy(GTK_WINDOW(mainWindow), FALSE, FALSE, FALSE);
	gtk_window_set_wmclass(GTK_WINDOW(mainWindow), _("GuiTuner"),
		_("Simple Guitar Tuner"));

	gtk_widget_realize(mainWindow);

	vBox = gtk_vbox_new(FALSE, 0);
	gtk_widget_ref(vBox);
	gtk_object_set_data_full(GTK_OBJECT(mainWindow), "vBox", vBox,
		(GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vBox);
	gtk_container_add(GTK_CONTAINER(mainWindow), vBox);

	initMenu();

	drawingArea = gtk_drawing_area_new();
	gtk_widget_ref(drawingArea);
	gtk_object_set_data_full(GTK_OBJECT(mainWindow), "drawingArea", drawingArea,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(drawingArea);
	gtk_box_pack_start(GTK_BOX(vBox), drawingArea, TRUE, TRUE, 0);

	logo = createPixmap(mainWindow, "GuiTuner/sign.xpm", FALSE);
	gtk_widget_ref(logo);
	gtk_object_set_data_full(GTK_OBJECT(mainWindow), "logo", logo,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(logo);
	gtk_box_pack_start(GTK_BOX(vBox), logo, FALSE, TRUE, 0);
	gtk_misc_set_alignment(GTK_MISC(logo), 0.9, 0.6);

	width = 400;
	height = 231;
	initDisplay();
	addHandlers();

	gtk_widget_show(mainWindow);
	isOpen = true;
}

void
Interface::GnomeGui::close() throw(Error::GuiError)
{
	if(loop)
		stop();
	if(isOpen)
		destroy(), isOpen = false;
}

void
Interface::GnomeGui::start() throw(Error::GuiError)
{
	loop = true;
	curr = Tuner::Note(0);
	tag = gtk_idle_add(idle_loop, this);
	gtk_main();
}

void
Interface::GnomeGui::stop() throw(Error::GuiError)
{
	loop = false;
	gtk_idle_remove(tag);
	gtk_main_quit();
}

void
Interface::GnomeGui::pause() throw(Error::GuiError)
{
	gtk_idle_remove(tag);
	curr = Tuner::Note(0);
	_display();
}

void
Interface::GnomeGui::resume() throw(Error::GuiError)
{
	tag = gtk_idle_add(idle_loop, this);
}

void
Interface::GnomeGui::delay(unsigned int ms) throw(Error::GuiError)
{
	usleep(ms);
}

void
Interface::GnomeGui::display(const Tuner::Note &note) throw(Error::GuiError)
{
	Thread::Lock lock(currLock);
	curr = note;
}

void
Interface::GnomeGui::display(const std::string &s) throw(Error::GuiError)
{
	_display(s, INFO);
}

void
Interface::GnomeGui::_display() throw()
{
	Thread::Lock lock(currLock);

	gdk_draw_pixmap(pix, drawingArea->style->fg_gc[GTK_WIDGET_STATE(drawingArea)], back,
		0, 0, 0, 0, width, height);

	int w = gdk_string_width(font, curr.getName());
	int	h = gdk_string_height(font, curr.getName());
	int	x = (width - w) / 2;
	int	y = height - 10 - h;

	gdk_draw_string(pix, font, drawingArea->style->white_gc, x, y, curr.getName());

	x = width / 2;
	y = height - 60;

	if(curr.getName()[0] != ' ') {
		if(curr.getCents() < 0) {
			gdk_draw_pixmap(pix, drawingArea->style->fg_gc[GTK_WIDGET_STATE(drawingArea)],
				file[3], 0, 0, x - 120, y, 50, 50);
		} else if(curr.getCents() > 0) {
			gdk_draw_pixmap(pix, drawingArea->style->fg_gc[GTK_WIDGET_STATE(drawingArea)],
				file[4], 0, 0, x + 70, y, 50, 50);
		}
		gdk_draw_line(pix, drawingArea->style->white_gc, ends[curr.getCents() + 50].x,
			ends[curr.getCents() + 50].y, ends[curr.getCents() + 50].x1,
			ends[curr.getCents() + 50].y1);
		lock.unlock();
	}

	gdk_draw_pixmap(drawingArea->window, drawingArea->style->fg_gc
		[GTK_WIDGET_STATE(drawingArea)], pix,
		0, 0, 0, 0, width, height);
}

void
Interface::GnomeGui::_display(const std::string &t, DialogType d) throw()
{
	GtkWidget *errorBox;
	GtkWidget *dialog_vbox1;
	GtkWidget *button3;
	GtkWidget *dialog_action_area1;

	errorBox = gnome_message_box_new(t.c_str(),
		d == ERROR ? GNOME_MESSAGE_BOX_ERROR: GNOME_MESSAGE_BOX_ERROR, 0);
	gtk_object_set_data(GTK_OBJECT(errorBox), "ErrorBox", errorBox);
	gtk_window_set_title(GTK_WINDOW(errorBox), d == ERROR ? _("Error"): _("Info"));
	gtk_window_set_policy(GTK_WINDOW(errorBox), FALSE, FALSE, FALSE);

	dialog_vbox1 = GNOME_DIALOG(errorBox)->vbox;
	gtk_object_set_data(GTK_OBJECT(errorBox), "dialog_vbox1", dialog_vbox1);
	gtk_widget_show(dialog_vbox1);

	gnome_dialog_append_button(GNOME_DIALOG(errorBox), GNOME_STOCK_BUTTON_OK);
	button3 = static_cast<GtkWidget *>(g_list_last(GNOME_DIALOG(errorBox)->buttons)->data);
	gtk_widget_ref(button3);
	gtk_object_set_data_full(GTK_OBJECT(errorBox), "button3", button3,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(button3);
	GTK_WIDGET_SET_FLAGS(button3, GTK_CAN_DEFAULT);

	dialog_action_area1 = GNOME_DIALOG(errorBox)->action_area;
	gtk_widget_ref(dialog_action_area1);
	gtk_object_set_data_full(GTK_OBJECT(errorBox), "dialog_action_area1",
		dialog_action_area1, (GtkDestroyNotify) gtk_widget_unref);

	gtk_widget_show(errorBox);
}

void
Interface::GnomeGui::expose(GdkEventExpose *event) throw()
{
	gdk_draw_pixmap(drawingArea->window,
		drawingArea->style->fg_gc[GTK_WIDGET_STATE(drawingArea)], pix,
		event->area.x, event->area.y, event->area.x, event->area.y,
		event->area.width, event->area.height);
}

void
Interface::GnomeGui::errorMsg(const std::string &s) throw()
{
	_display(s, ERROR);
}

void
Interface::onExitActivate(GtkMenuItem *, gpointer)
{
	GuiTuner::stop();
}

void
Interface::onPreferencesActivate(GtkMenuItem *, gpointer)
{
	openPrefWindow();
}

void
Interface::onAboutActivate(GtkMenuItem *, gpointer)
{
	const gchar *authors[] = {
		"Fabio Checconi <fchecconi@libero.it>",
		NULL
	};
	GtkWidget *aboutBox;

	aboutBox = gnome_about_new("GuiTuner", VERSION,
		_("Copyright Fabio Checconi (c) 2001"),
		authors,
		_("Simple Guitar Tuning Utility"),
		"GuiTuner/logo.xpm");

	gtk_object_set_data(GTK_OBJECT(aboutBox), "aboutBox", aboutBox);
	gtk_container_set_border_width(GTK_CONTAINER (aboutBox), 2);
	gtk_window_set_modal (GTK_WINDOW (aboutBox), TRUE);

	gtk_widget_show(aboutBox);
}

gint
Interface::delete_event(GtkWidget *, GdkEvent *, gpointer)
{
	GuiTuner::stop();
	return TRUE;
}

gint
Interface::expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
	GnomeGui *gui = static_cast<GnomeGui *>(user_data);

	gui->expose(event);

	return FALSE;
}

gint
Interface::idle_loop(gpointer user_data)
{
	GnomeGui *gui = static_cast<GnomeGui *>(user_data);

	gui->delay(500);
	gui->refresh();

	return TRUE;
}

