/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h> 
#endif

#include <iostream>
#include <stdexcept>
#include <string>
#include <unistd.h>
#include "error.h"
#include "gnomegui.h"
#include "textgui.h"
#include "guituner.h"

GuiTuner *GuiTuner::obj = 0;

GuiTuner::GuiTuner(int &argc, char **&argv):
	gui(0), mt(0), conf(0)
{
	using namespace Interface;

	try {
#ifdef ENABLE_NLS
		bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
		textdomain(PACKAGE);
#endif

		gui = new GnomeGui(&argc, &argv);
	} catch(...) {
		gui = new TextGui(&argc, &argv);
	}

	try {
		conf = new Config::GenConf(argc, argv);
		gui->open();
		mt = conf->createMainThread(gui);
	} catch(Error::Error &e) {
		gui->errorMsg(std::string(_("Cannot start application ")) + e.getFullMsg());
		exit(-1);
	} catch(std::exception &e) {
		gui->errorMsg(std::string(_("Cannot start application ")) + e.what());
		exit(-1);
	}

	obj = this;
}

GuiTuner::~GuiTuner()
{
	if(obj->mt->isRunning())
		stop();

	delete conf;
	delete gui;

	obj = 0;
}

void
GuiTuner::start()
{
	try {
		obj->mt->run();
		obj->gui->start();
	} catch(Error::Error &e) {
		obj->gui->errorMsg(std::string(_("Cannot start threads ")) + e.getFullMsg());
		exit(-1);
	} catch(std::exception &e) {
		obj->gui->errorMsg(std::string(_("Cannot start threads ")) + e.what());
		exit(-1);
	}
}

void
GuiTuner::stop()
{
	using namespace Thread;

	try {
		MainThread::Command comm = MainThread::STOP;
		obj->mt->sendMsg(&comm, sizeof(MainThread::Command));
		obj->mt->wait();

		obj->gui->stop();
	} catch(Error::Error &e) {
		obj->gui->errorMsg(std::string(_("Cannot stop threads ")) + e.getFullMsg());
		exit(-1);
	} catch(std::exception &e) {
		obj->gui->errorMsg(std::string(_("Cannot stop threads ")) + e.what());
		exit(-1);
	}
}

void
GuiTuner::pause()
{
	using namespace Thread;

	try {
		MainThread::Command comm = MainThread::PAUSE;
		obj->mt->sendMsg(&comm, sizeof(MainThread::Command));

		obj->gui->pause();
	} catch(Error::Error &e) {
		obj->gui->errorMsg(std::string(_("Cannot pause threads ")) + e.getFullMsg());
		exit(-1);
	} catch(std::exception &e) {
		obj->gui->errorMsg(std::string(_("Cannot pause threads ")) + e.what());
		exit(-1);
	}
}

void
GuiTuner::resume()
{
	using namespace Thread;

	try {
		obj->gui->resume();

		MainThread::Command comm = MainThread::CONTINUE;
		obj->mt->sendMsg(&comm, sizeof(MainThread::Command));
	} catch(Error::Error &e) {
		obj->gui->errorMsg(std::string(_("Cannot restart application ")) + e.getFullMsg());
		exit(-1);
	} catch(std::exception &e) {
		obj->gui->errorMsg(std::string(_("Cannot restart application ")) + e.what());
		exit(-1);
	}
}

void 
GuiTuner::reconfigure(Tuner::AudioDriver ad, Tuner::DetectionMode dm,
	Tuner::AudioInfo *ai, Tuner::CalcInfo *ci, Tuner::NoteInfo *ni)
{
	using namespace Thread;

	try {
		MainThread::Command comm = MainThread::STOP;
		obj->mt->sendMsg(&comm, sizeof(MainThread::Command));
		obj->mt->wait();

		obj->conf->setInfo(ad, dm, ai, ci, ni);
		obj->mt = obj->conf->createMainThread(obj->gui);

		obj->mt->run();
	} catch(Error::Error &e) {
		obj->gui->errorMsg(std::string(_("Cannot configure application "))
			+ e.getFullMsg());
		exit(-1);
	} catch(std::exception &e) {
		obj->gui->errorMsg(std::string(_("Cannot configure application "))
			 + e.what());
		exit(-1);
	}
}

Config::GenConf *
GuiTuner::getConf() throw()
{
	return obj->conf;
}

void
GuiTuner::reportMsg(const char *msg) throw()
{
	if(obj)
		obj->gui->errorMsg(msg);
	else
		std::cerr << msg << endl;
}

