/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cmath>
#include "note.h"

namespace Tuner {
	const char *NoteName[CONV_NUMBER][14] = {
		{ "A ", "A#", "B ", "C ", "C#", "D ", "D#", "E ", "F ", "F#", "G ",
			"G#", "A ", "  " },
		{ "La  ", "La# ", "Si  ", "Do  ", "Do# ", "Re  ", "Re# ", "Mi  ",
			"Fa  ", "Fa# ", "Sol ", "Sol#", "La  ",  "    " }
	};

	const int BlankIndex = 13;
	const double NFact = 1.059463094;
}

Tuner::NamingConv Tuner::Note::conv = ENGLISH;
double Tuner::Note::ref = 440.0;
double Tuner::Note::delta = 2.0;

Tuner::Note::Note(double f) throw()
{
	if(f <= 8) {
		nameIndex = BlankIndex;
		cents = -49;
		return;
	}

	double e = ref;
	if(e > f) {
		while(e > f)
			e /= 2;
	} else {
		while(e < f / 2)
			e *= 2;
	}

	double d = f / e;
	double v = 12 * std::log(d) / std::log(2);
	nameIndex = static_cast<int>(std::rint(v));
	nameIndex %= 13;

	cents = static_cast<int>((v - nameIndex) * 100);
	for(int k = 0; k < nameIndex; ++k)
		e *= NFact;
	e -= f;
	v = delta / 2;
	if(e <= v && e >= -v)
		cents = 0;
}

const char *
Tuner::Note::getName() const throw()
{
	return NoteName[conv][nameIndex];
}

