/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <gnome.h>
#include "support.h"

namespace Interface {
	GtkWidget* createDummyPixmap(GtkWidget *widget, gboolean gnomePixmap);

	char *dummyPixmapXpm[] = {
		"1 1 1 1",
		"  c None",
		" ",
		" "
	};
}

GtkWidget*
Interface::lookupWidget(GtkWidget *widget, const gchar *widgetName)
{
	GtkWidget *parent, *foundWidget;

	for(;;) {
		if(GTK_IS_MENU(widget))
			parent = gtk_menu_get_attach_widget(GTK_MENU(widget));
		else
			parent = widget->parent;
		if(parent == NULL)
			break;
		widget = parent;
	}

	foundWidget = static_cast<GtkWidget*>(gtk_object_get_data(GTK_OBJECT(widget),
		widgetName));

	if(!foundWidget)
    g_warning(_("Widget not found: %s"), widgetName);

	return foundWidget;
}

GtkWidget*
Interface::createDummyPixmap(GtkWidget *widget, gboolean gnomePixmap)
{
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	GtkWidget *pixmap;

	if(gnomePixmap)
		return gnome_pixmap_new_from_xpm_d(dummyPixmapXpm);

	colormap = gtk_widget_get_colormap(widget);
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap, &mask,
		NULL, dummyPixmapXpm);
	if(gdkpixmap == NULL)
		g_error(_("Couldn't create replacement pixmap."));
	pixmap = gtk_pixmap_new(gdkpixmap, mask);
	gdk_pixmap_unref(gdkpixmap);
	gdk_bitmap_unref(mask);

	return pixmap;
}

GtkWidget*
Interface::createPixmap(GtkWidget *widget, const gchar *filename, gboolean gnomePixmap)
{
	GtkWidget *pixmap;
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	gchar *pathname;

	if(!filename || !filename[0])
		return createDummyPixmap(widget, gnomePixmap);

	pathname = gnome_pixmap_file(filename);
	if(!pathname) {
		g_warning(_("Couldn't find pixmap file: %s"), filename);

		return createDummyPixmap(widget, gnomePixmap);
    }

	if(gnomePixmap) {
		pixmap = gnome_pixmap_new_from_file(pathname);
		g_free(pathname);

		return pixmap;
	}

	colormap = gtk_widget_get_colormap(widget);
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm(NULL, colormap, &mask,
		NULL, pathname);
	if (gdkpixmap == NULL) {
		g_warning(_("Couldn't create pixmap from file: %s"), pathname);
		g_free (pathname);

		return createDummyPixmap(widget, gnomePixmap);
    }

	g_free(pathname);

	pixmap = gtk_pixmap_new(gdkpixmap, mask);
	gdk_pixmap_unref(gdkpixmap);
	gdk_bitmap_unref(mask);

	return pixmap;
}

GdkImlibImage*
Interface::createImage(const gchar *filename)
{
	GdkImlibImage *image;
	gchar *pathname;

	pathname = gnome_pixmap_file(filename);
	if(!pathname) {
		g_warning(_("Couldn't find pixmap file: %s"), filename);

		return NULL;
    }

	image = gdk_imlib_load_image(pathname);
	g_free(pathname);

	return image;
}

