/*  GuiTuner 0.05
 *  Copyright (c) 2001 Fabio Checconi <fchecconi@libero.it>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include "textgui.h"
#include "i18n.h"

void
Interface::TextGui::sigHandler(int sig) throw()
{
	std::cerr << _("Received signal ") << sig << _("\nExit\n");
	exit(-1);
}

Interface::TextGui::TextGui(int *argc, char ***argv) throw(Error::GuiError):
	stopReq(false)
{
	for(int i = 0; i < 16; ++i)
		signal(i, TextGui::sigHandler);
}

Interface::TextGui::~TextGui()
{
	stop();
}

void
Interface::TextGui::open() throw(Error::GuiError)
{
	std::cout << _("GuiTuner 0.05 - Copyright (c) Fabio Checconi 2001\n");
	std::cout << _(" Simple guitar tuner - press ENTER to quit.\n\n");
}

void
Interface::TextGui::close() throw(Error::GuiError)
{
}

void
Interface::TextGui::start() throw(Error::GuiError)
{
	stopReq = false;

	while(!stopReq) {
		timeval timeOut = { 0, 500000 };
		fd_set set1, set2, set3;

		FD_ZERO(&set1);
		FD_ZERO(&set2);
		FD_ZERO(&set3);
		FD_SET(STDIN_FILENO, &set1);
		if(select(STDIN_FILENO + 1, &set1, &set2, &set3, &timeOut) != 0) {
			char ch;
			read(STDIN_FILENO, &ch, sizeof(char));
			break;
		}
	}
}

void
Interface::TextGui::stop() throw(Error::GuiError)
{
	stopReq = true;
}

void
Interface::TextGui::pause() throw(Error::GuiError)
{
	stop();
}

void
Interface::TextGui::resume() throw(Error::GuiError)
{
	start();
}

void
Interface::TextGui::delay(unsigned int ms) throw(Error::GuiError)
{
	usleep(ms);
}

void
Interface::TextGui::display(const Tuner::Note &note) throw(Error::GuiError)
{
	std::cout << "==>> " << note.getName() << " " << note.getCents() << std::endl;
}

void
Interface::TextGui::display(const std::string &msg) throw(Error::GuiError)
{
	std::cout << msg << '\n';
}

void
Interface::TextGui::errorMsg(const std::string &msg) throw()
{
	std::cerr << msg << std::endl;
}

